import * as sync from "./../../src/utils/sync-utils";
import {initTests} from "./../globals";
import {DEFAULT_NFS_VERSION} from './../../src/consts'

beforeEach(() => {
    initTests();
});


describe("breakFullPathToRootAndPath", () => {
    const paths = "folder1/folder2/folder3/folder4";
    const root = "/folder1/folder2/folder3";
    test("should return the same paths", () => {
        expect(sync.breakFullPathToRootAndPath(paths, "folder2")).toEqual({
            "pathWithoutRoot": "folder1/folder2/folder3/folder4",
            "rootOfPath": "folder2"
        });
    });
    test("should return the same paths", () => {
        expect(sync.breakFullPathToRootAndPath(paths, "")).toEqual({
            "pathWithoutRoot": "folder1/folder2/folder3/folder4",
            "rootOfPath": ""
        });
    });
    test("should return the same paths", () => {
        expect(sync.breakFullPathToRootAndPath("", root)).toEqual({
            "pathWithoutRoot": "",
            "rootOfPath": "/folder1/folder2/folder3"
        });
    });
    test("should return [/folder1/folder2/folder3]", () => {
        expect(sync.breakFullPathToRootAndPath("folder1/folder2/folder3", root)).toEqual({
            "pathWithoutRoot": "",
            "rootOfPath": "/folder1/folder2/folder3"
        });
    });
    test("should return [folder1/folder2/folder3, folder4]", () => {
        expect(sync.breakFullPathToRootAndPath(paths, root)).toEqual({
            "pathWithoutRoot": "folder4",
            "rootOfPath": "/folder1/folder2/folder3"
        });
    });


});

describe("appendItemToLocal", () => {
    const domain = "guy", username = "test1", password = "123456", ip = "1.1.1.1";
    let cred = {domain, username, password};
    let retCred = {domain, username, password};
    test("should return the cred", () => {
        sync.appendItemToLocalStorage("cifs", ip, cred);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
    });
    test("should overwrite the username", () => {
        cred = {...cred, username: "test2"};
        retCred = {...retCred, username: "test2"};
        sync.appendItemToLocalStorage("cifs", ip, cred);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
    });
    test("should return both of the cred", () => {
        const cred2 = {...cred, username: "test4"};
        const retCred2 = {...retCred, username: "test4"};
        sync.appendItemToLocalStorage("cifs", ip, cred);
        sync.appendItemToLocalStorage("cifs", "2.2.2.2", cred2);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
        expect(sync.getCredentialsByType("cifs", "2.2.2.2")).toEqual(retCred2);
    });
    test("should return both cifs and sg", () => {
        const cred2 = {...cred, username: "test4"};
        const retCred2 = {...retCred, username: "test4"};
        const sgCred = {accessKey: "accessKey", secretKey: "secretKey", port: "80"};
        const sgCred2 = {accessKey: "accessKey2", secretKey: "secretKey2", port: "88"};
        sync.appendItemToLocalStorage("cifs", ip, cred);
        sync.appendItemToLocalStorage("cifs", "2.2.2.2", cred2);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
        expect(sync.getCredentialsByType("cifs", "2.2.2.2")).toEqual(retCred2);
        sync.appendItemToLocalStorage("objectStorage", ip, sgCred);
        sync.appendItemToLocalStorage("objectStorage", "2.2.2.2", sgCred2);
        expect(sync.getCredentialsByType("objectStorage", ip)).toEqual(sgCred);
        expect(sync.getCredentialsByType("objectStorage", "2.2.2.2")).toEqual(sgCred2);
    });
});


describe('isSourceTargetSupported', () => {
    test('should return true on supported combination', function () {
        expect(sync.isSourceTargetSupported({type: "nfs"}, {type: "s3"})).toEqual(true);
    });
    test('should return false on unsupported combination', function () {
        expect(sync.isSourceTargetSupported({type: "nfs"}, {type: "cifs"})).toEqual(false);
    });
});

describe("protocolByWizardType", () => {
    test("should return the right protocols for the requested wizard type", () => {
        expect(sync.protocolByWizardType("s3-nfs")).toEqual({source: "s3", target: "nfs"});
    });

    test("should fix the sg to the right protocol", () => {
        expect(sync.protocolByWizardType("nfs-sgws")).toEqual({source: "nfs", target: "sgws"});
    });
});

describe("getCredentials", () => {
    const saveCredentials = (key, innerKey, value) => {
        window.localStorage.setItem(key, JSON.stringify({[innerKey]: value}));
    };
    test("should return the right CIFS credentials when exist", () => {
        const innerKey = "1.1.1.1";
        const item = {domain: "domain", username: "user", password: "password"};
        saveCredentials("cifs", innerKey, item);
        expect(sync.getCredentialsByType("cifs", "1.1.1.1")).toEqual({
            domain: "domain",
            username: "user",
            password: "password"
        });
    });

    test("should return the right SG credentials when exist", () => {
        const item = {port: "80", secretKey: "secretKey", accessKey: "accessKey"};
        const key = "1.1.1.1";
        saveCredentials("storageGrid", key, item);
        const serverCredentials = sync.getCredentialsByType("storageGrid", key);
        expect(serverCredentials).toEqual(item);
    });

});

describe("buildSyncObject", () => {
    const fillSGCredentials = (sg) => {
        const secretKey = "secretKey";
        const accessKey = "accessKey";
        const port = "80";
        sync.appendItemToLocalStorage("storageGrid", sg, {port, secretKey, accessKey});
    };

    const fillCIFSCredentials = (cifs, isTargetOfSameSource) => {
        const username = "username";
        const password = "password";
        const domain = "domain";
        sync.appendItemToLocalStorage("cifs", cifs, {domain, username, password}, isTargetOfSameSource);
    };

    const fillAzureCredentials = (storageAccountName) => {
        const storageAccountConnectionString = "storageAccountConnectionString";
        sync.appendItemToLocalStorage("azure", storageAccountName, {storageAccountName, connectionString: storageAccountConnectionString});
    };


    test("should build object for nfs to nfs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "nfs-nfs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }
        })).toEqual({
                additionalInfo: {gracePeriod: 30, deleteOnTarget: false},
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                sourceCredentials: undefined,
                syncInDays: 1,
                syncInHours: 0,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });

    test("should build object for nfs to cvs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "nfs-cvs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }
        })).toEqual({
                additionalInfo: {gracePeriod: 30, deleteOnTarget: false},
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                sourceCredentials: undefined,
                syncInDays: 1,
                syncInHours: 0,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.CVS, "nfsType": "cvs"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });
    test("should build object for nfs to anf relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "nfs-anf",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }
        })).toEqual({
                additionalInfo: {gracePeriod: 30, deleteOnTarget: false},
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                sourceCredentials: undefined,
                syncInDays: 1,
                syncInHours: 0,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.ANF, "nfsType": "anf"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });
    test("should build object for cvs to nfs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "cvs-nfs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }
        })).toEqual({
                additionalInfo: {gracePeriod: 30, deleteOnTarget: false},
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.CVS, "nfsType": "cvs"}, protocol: "nfs"},
                sourceCredentials: undefined,
                syncInDays: 1,
                syncInHours: 0,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });
    test("should build object for anf to nfs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "anf-nfs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }
        })).toEqual({
                additionalInfo: {gracePeriod: 30, deleteOnTarget: false},
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.ANF, "nfsType": "anf"}, protocol: "nfs"},
                sourceCredentials: undefined,
                syncInDays: 1,
                syncInHours: 0,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });

    test("should build object for nfs to efs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "nfs-efs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1/folder1",
                    selectedExportTarget: "/folder2",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "folder1", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"nfs": {"export": "/", "host": "2.2.2.2", "path": "folder2", "nfsType": "efs", "version": DEFAULT_NFS_VERSION.EFS}, "protocol": "nfs"},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for nfs to s3 relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "nfs-s3",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    relationshipTags: "a=b",
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/data1",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1", "tags": [{"key": "a", "value": "b"}]}},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for nfs to sg relationship", () => {
        const sg = "2.2.2.2";
        fillSGCredentials(sg);

        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "nfs-sgws",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedGracePeriod: "30",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedSG: sg,
                    selectedSGBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedSGBucketPath: "large/targetBucket"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {
                    "protocol": "sgws",
                    "sgws": {"bucket": "large", "host": "2.2.2.2", objectStorageType: "sgws", "port": "88", "prefix": "targetBucket"}
                },
                "targetCredentials": {"sgws": {"accessKey": "accessKey2", "secretKey": "secretKey2"}}
            }
        );
    });
    test("should build object for efs to nfs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "efs-nfs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/folder1",
                    selectedExportTarget: "/data1",
                    selectedGracePeriod: "30",
                    selectedNFS: "2.2.2.2",
                    selectedNFSTarget: "1.1.1.1",
                    selectedRootExport: "/",
                    selectedRootExportTarget: "/data1"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/", "host": "2.2.2.2", "path": "folder1", "nfsType": "efs", "version": DEFAULT_NFS_VERSION.EFS}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, "protocol": "nfs"},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for cifs to cifs relationship", () => {
        fillCIFSCredentials("1.1.1.1");
        fillCIFSCredentials("2.2.2.2", true);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "cifs-cifs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedCIFS: "1.1.1.1",
                    selectedCIFSTarget: "2.2.2.2",
                    selectedGracePeriod: "30",
                    selectedRootShare: "/data1",
                    selectedRootShareTarget: "/data2",
                    selectedShare: "/data1/folder1",
                    selectedShareTarget: "/data2/folder1/targetFolder1"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"cifs": {"host": "1.1.1.1", "path": "folder1", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "sourceCredentials": {"cifs": {username: "username", password: "password", domain: "domain"}},
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {
                    "cifs": {"host": "2.2.2.2", "path": "folder1/targetFolder1", "share": "/data2", version: "2.1"}, "protocol": "cifs"
                },
                "targetCredentials": {"cifs": {username: "username", password: "password", domain: "domain"}}
            }
        );
    });
    test("should build object for cifs to s3 relationship", () => {
        fillCIFSCredentials("1.1.1.1");
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "cifs-s3",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "large/targetBucket",
                    selectedCIFS: "1.1.1.1",
                    selectedGracePeriod: "30",
                    selectedRootShare: "/data1",
                    selectedShare: "/data1"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"cifs": {"host": "1.1.1.1", "path": "", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "sourceCredentials": {"cifs": {domain: "domain", username: "username", password: "password"}},
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"protocol": "s3", "s3": {"bucket": "large", "prefix": "targetBucket"}},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for cifs to sg relationship", () => {
        const sg = "2.2.2.2";
        fillSGCredentials(sg);
        const cifs = "1.1.1.1";
        fillCIFSCredentials(cifs);

        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "cifs-sgws",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedCIFS: cifs,
                    selectedGracePeriod: "30",
                    selectedRootShare: "/data1",
                    selectedSG: sg,
                    selectedSGBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedSGBucketPath: "large",
                    selectedShare: "/data1/folder1"
                }
            }
        })).toEqual({
                "additionalInfo": {"gracePeriod": 30, deleteOnTarget: false},
                "dataBrokerId": 1,
                "source": {"cifs": {"host": "1.1.1.1", "path": "folder1", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "sourceCredentials": {"cifs": {domain: "domain", username: "username", password: "password"}},
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"protocol": "sgws", "sgws": {"bucket": "large", "host": "2.2.2.2", objectStorageType: "sgws", "port": "88", "prefix": ""}},
                "targetCredentials": {"sgws": {"accessKey": "accessKey2", "secretKey": "secretKey2"}}
            }
        );
    });
    test("should build object for s3 to nfs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "s3-nfs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/complex/junction/path",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: true,
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/complex/junction/path"
                }
            }
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1"}},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"nfs": {"export": "/complex/junction/path", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "additionalInfo": {"deleteOnTarget": true, "gracePeriod": 30}
            }
        );
    });

    test("should build object for s3 to cvs relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "s3-cvs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/complex/junction/path",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: true,
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/complex/junction/path"
                }
            }
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1"}},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"nfs": {"export": "/complex/junction/path", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.CVS, "nfsType": "cvs"}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "additionalInfo": {"deleteOnTarget": true, "gracePeriod": 30}
            }
        );
    });
    test("should build object for s3 to anf relationship", () => {
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "s3-anf",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/complex/junction/path",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: true,
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/complex/junction/path"
                }
            }
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1"}},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"nfs": {"export": "/complex/junction/path", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.ANF, "nfsType": "anf"}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "additionalInfo": {"deleteOnTarget": true, "gracePeriod": 30}
            }
        );
    });

    test("should build object for s3 to cifs relationship", () => {
        const cifs = "1.1.1.1";
        fillCIFSCredentials(cifs);

        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "s3-cifs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler",
                    selectedCIFS: cifs,
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: true,
                    selectedRootShare: "/data1",
                    selectedShare: "/data1"
                }
            }
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": ""}},
                "sourceCredentials": undefined,
                "syncInDays": 1,
                "syncInHours": 0,
                "target": {"cifs": {"host": "1.1.1.1", "path": "", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "targetCredentials": {"cifs": {domain: "domain", username: "username", password: "password"}},
                "additionalInfo": {"deleteOnTarget": true, "gracePeriod": 30},
            }
        );
    });
    test("should build object for azure to nfs relationship", () => {
        fillAzureCredentials("storageAccountName");
        expect(sync.buildSyncObject({
            _wizard: {
                wizardType: "azure-nfs",
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: true,
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedStorageAccountName: "storageAccountName",
                    selectedAzureBlobContainerPath: "container/test",
                    selectedBlobTier: "COOL"
                }
            }
        })).toEqual({
                dataBrokerId: 1,
                source: {
                    protocol: "azure",
                    azure: {storageAccountName: "storageAccountName", container: "container", prefix: "test","tags": [], blobTier: "COOL"}
                },
                sourceCredentials: {azure: {storageAccountConnectionString: "storageAccountConnectionString"}},
                syncInDays: 1,
                syncInHours: 0,
                target: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS, "nfsType": "nfs"}, protocol: "nfs"},
                additionalInfo: {deleteOnTarget: true, gracePeriod: 30},
            }
        );
    });
});

describe("getSourceAndTargetInfo", () => {

    test("should build object for nfs to nfs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "nfs-nfs",
                selectedWizardParams: {
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server",
                },
                "targetInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data2",
                    "title": "NFS Server",
                }
            }
        );
    });
    test("should build object for nfs to efs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "nfs-efs",
                selectedWizardParams: {
                    selectedExport: "/data1/folder1",
                    selectedExportTarget: "/folder2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                }
            }
        })).toEqual(
            {
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1/folder1",
                    "title": "NFS Server"
                },
                "targetInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Directory",
                    "subtitleDescription": "/folder2",
                    "title": "AWS EFS"
                }
            }
        );
    });
    test("should build object for nfs to s3 relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "nfs-s3",
                selectedWizardParams: {
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server"
                },
                "targetInfo": {
                    "description": "chandler",
                    "subtitle": "Folder",
                    "subtitleDescription": "centralPark1",
                    "title": "AWS S3 Bucket"
                }
            }
        );
    });
    test("should build object for nfs to sg relationship", () => {
        const sg = "2.2.2.2";
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "nfs-sgws",
                selectedWizardParams: {
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedSG: sg,
                    selectedSGBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedSGBucketPath: "large/targetBucket"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server"
                },
                "targetInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "targetBucket",
                    "title": "StorageGRID Bucket"
                }
            }
        );
    });
    test("should build object for efs to nfs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "efs-nfs",
                selectedWizardParams: {
                    selectedExport: "/folder1",
                    selectedExportTarget: "/data1",
                    selectedNFS: "2.2.2.2",
                    selectedNFSTarget: "1.1.1.1"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Directory",
                    "subtitleDescription": "/folder1",
                    "title": "AWS EFS"
                },
                "targetInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server"
                }
            }
        );
    });
    test("should build object for cifs to cifs relationship", () => {

        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "cifs-cifs",
                selectedWizardParams: {
                    selectedCIFS: "1.1.1.1",
                    selectedCIFSTarget: "2.2.2.2",
                    selectedShare: "/data1/folder1",
                    selectedShareTarget: "/data2/folder1/targetFolder1"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1/folder1",
                    "title": "CIFS Server"
                },
                "targetInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Share",
                    "subtitleDescription": "/data2/folder1/targetFolder1",
                    "title": "CIFS Server"
                }
            }
        );
    });
    test("should build object for cifs to s3 relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "cifs-s3",
                selectedWizardParams: {
                    selectedBucketPath: "large/targetBucket",
                    selectedCIFS: "1.1.1.1",
                    selectedShare: "/data1"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1",
                    "title": "CIFS Server"
                },
                "targetInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "targetBucket",
                    "title": "AWS S3 Bucket"
                }
            }
        );
    });
    test("should build object for cifs to sg relationship", () => {
        const sg = "2.2.2.2";
        const cifs = "1.1.1.1";

        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "cifs-sgws",
                selectedWizardParams: {
                    selectedCIFS: cifs,
                    selectedRootShare: "/data1",
                    selectedSG: sg,
                    selectedSGBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedSGBucketPath: "large",
                    selectedShare: "/data1/folder1"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1/folder1",
                    "title": "CIFS Server"
                },
                "targetInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "StorageGRID Bucket"
                }
            }
        );
    });
    test("should build object for s3 to nfs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "s3-nfs",
                selectedWizardParams: {
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExportTarget: "/complex/junction/path",
                    selectedNFSTarget: "1.1.1.1",
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "chandler",
                    "subtitle": "Folder",
                    "subtitleDescription": "centralPark1",
                    "title": "AWS S3 Bucket"
                },
                "targetInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/complex/junction/path",
                    "title": "NFS Server"
                }
            }
        );
    });
    test("should build object for s3 to sgws relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "s3-sgws",
                selectedWizardParams: {
                    selectedSG: "123.123.123.123",
                    selectedSGBucketPath: "small",
                    selectedBucketPath: "tiny",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: false,
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "tiny",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "AWS S3 Bucket"
                },
                "targetInfo": {
                    "description": "small",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "StorageGRID Bucket"
                }
            }
        );
    });
    test("should build object for sgws to s3 relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "sgws-s3",
                selectedWizardParams: {
                    selectedSG: "123.123.123.123",
                    selectedSGBucketPath: "small",
                    selectedBucketPath: "tiny",
                    selectedStorageClass: "STANDARD",
                    relationshipTags: "noTags",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: false,
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "small",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "StorageGRID Bucket"
                },
                "targetInfo": {
                    "description": "tiny",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "AWS S3 Bucket"
                }
            }
        );
    });
    test("should build object for cifs to gcp relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "cifs-gcp",
                selectedWizardParams: {
                    selectedCIFS: "123.123.123.123",
                    selectedShare: "/data2",
                    cifsVersion: "2.1",
                    selectedRootShare: "/data2",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: false,
                    selectedGCPBucketPath: "large",
                    selectedGCPProjectId: "uniqueProjectId"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "123.123.123.123",
                    "subtitle": "Share",
                    "subtitleDescription": "/data2",
                    "title": "CIFS Server"
                },
                "targetInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "GCP Storage Bucket"
                }
            }
        );
    });
    test("should build object for gcp to cifs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "gcp-cifs",
                selectedWizardParams: {
                    selectedCIFS: "123.123.123.123",
                    selectedShare: "/data2",
                    cifsVersion: "2.1",
                    selectedRootShare: "/data2",
                    selectedGracePeriod: "30",
                    selectedDeleteOnTarget: false,
                    selectedGCPBucketPath: "large",
                    selectedGCPProjectId: "uniqueProjectId"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "GCP Storage Bucket"
                },
                "targetInfo": {
                    "description": "123.123.123.123",
                    "subtitle": "Share",
                    "subtitleDescription": "/data2",
                    "title": "CIFS Server"
                }
            }
        );
    });
    test("should build object for s3 to cifs relationship", () => {
        const cifs = "1.1.1.1";

        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardType: "s3-cifs",
                selectedWizardParams: {
                    selectedBucketPath: "chandler",
                    selectedCIFSTarget: cifs,
                    selectedGracePeriod: "30",
                    selectedShareTarget: "/data1"
                }
            }
        })).toEqual({
                "sourceInfo": {
                    "description": "chandler",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "AWS S3 Bucket"
                },
                "targetInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1",
                    "title": "CIFS Server"
                }
            }
        );
    });
});

describe("getWizardSelectedParams", () => {
    test("should build the right selected params for nfs-s3", () => {

        const nfsServer = "1---252E1---252E1---252E1";
        const exportPath = "---252Fdata1---252Ffolder1";
        const rootExport = "---252Fdata1";
        const dataBrokerId = 1;
        const bucketPath = "chandler---252FcentralPark1";
        const relationshipTags = "test---3D1";
        const gracePriod = 30;
        const wizardParams = {
            nfsServer,
            exportPath,
            rootExport,
            dataBrokerId,
            bucketPath,
            relationshipTags,
            gracePriod
        };

        const props = {
            _wizard: {
                wizardType: "nfs-s3"
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showExports: {
                [nfsServer]: {
                    succeeded: true,
                    data: []
                }
            },
            _showBuckets: {
                [dataBrokerId]: {
                    succeeded: true,
                    data: [{name: "chandler"}]
                }

            }

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                relationshipTags: "test=1",
                selectedNFS: "1.1.1.1",
                selectedExport: "/data1/folder1",
                selectedRootExport: "/data1",
                selectedGracePeriod: "30",
                selectedDeleteOnTarget: false,
                selectedBucketPath: "chandler/centralPark1"
            }
        );
    });
    test("should build the right selected params for cifs-s3", () => {

        const cifsServer = "1---252E1---252E1---252E1";
        const sharePath = "---252Fdata1---252Ffolder1";
        const rootShare = "---252Fdata1";
        const dataBrokerId = 1;
        const bucketPath = "chandler---252FcentralPark1";
        const gracePriod = 30;
        const wizardParams = {cifsServer, sharePath, rootShare, dataBrokerId, bucketPath, gracePriod};

        const props = {
            _wizard: {
                wizardType: "cifs-s3"
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showShares: {
                [cifsServer]: {
                    succeeded: true,
                    data: []
                }
            },
            _showBuckets: {
                [dataBrokerId]: {
                    succeeded: true,
                    data: [{name: "chandler"}]
                }
            }

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                selectedCIFS: "1.1.1.1",
                selectedRootShare: "/data1",
                selectedShare: "/data1/folder1",
                selectedGracePeriod: "30",
                selectedDeleteOnTarget: false,
                selectedBucketPath: "chandler/centralPark1"
            }
        );
    });
    test("should build the right selected params for cifs-sgws", () => {

        const cifsServer = "1---252E1---252E1---252E1";
        const dataBrokerId = "1";
        const wizardParams = {
            cifsServer,
            dataBrokerId,
            gracePeriod: "30",
            rootShare: "---252Fdata1",
            sgBucketPath: "large",
            sgServer: "2---252E2---252E2---252E2",
            shareExport: "---252Fdata1",
            sharePath: "---252Fdata1---252Ffolder1",
            deletionOn: "true"
        };

        const props = {
            _wizard: {
                wizardType: "cifs-sgws"
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showShares: {
                [cifsServer]: {
                    succeeded: true,
                    data: []
                }
            },
            _showSGBuckets: { '2.2.2.2': {
                succeeded: true,
                data: [{name: "large"}]
            }}

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                selectedCIFS: "1.1.1.1",
                selectedRootShare: "/data1",
                selectedShare: "/data1/folder1",
                selectedGracePeriod: "30",
                selectedDeleteOnTarget: true,
                selectedSG: "2.2.2.2",
                selectedSGBucket: {name: "large"},
                selectedSGBucketPath: "large"
            }
        );
    });
    test("should build the right selected params for efs-nfs", () => {

        const exportPath = "---252Ffolder1---252Ffolder0";
        const exportPathTarget = "---252Fcomplex---252Fjunction---252Fpath---252Ftest";
        const gracePeriod = "30";
        const nfsServer = "1---252E1---252E1---252E1";
        const nfsServerTarget = "2---252E2---252E2---252E2";
        const rootExport = "---252F";
        const rootExportTarget = "---252Fcomplex---252Fjunction---252Fpath";
        const dataBrokerId = 1;
        const wizardParams = {
            exportPath,
            exportPathTarget,
            nfsServer,
            nfsServerTarget,
            dataBrokerId,
            rootExport,
            rootExportTarget,
            gracePeriod
        };

        const props = {
            _wizard: {
                wizardType: "efs-nfs"
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showExports: {
                [nfsServer]: {
                    succeeded: true,
                    data: []
                },
                [nfsServerTarget]: {
                    succeeded: true,
                    data: []
                }
            }

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                selectedNFS: "1.1.1.1",
                selectedNFSTarget: "2.2.2.2",
                selectedRootExport: "/",
                selectedExport: "/folder1/folder0",
                selectedGracePeriod: "30",
                selectedDeleteOnTarget: false,
                selectedExportTarget: "/complex/junction/path/test",
                selectedRootExportTarget: "/complex/junction/path"
            }
        );
    });
});